package bid.ex.support;

import no.bbs.bankid.netcentric.centralverifier.communication.CSerialSigningData;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.eclipse.jetty.util.StringUtil;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.List;
import java.util.Optional;

/**
 * Class for holding state between steps for BankID Web Client or BankID mobil
 */
public class SessionSequenceInfo {

    private NextBankIDStep nextStep;
    private String errorMsg;
    private String errorCode = "0";

    private final String sid;
    private String ssn;

    // Signing data
    private List<ParallelOrSerialDocumentToSign> dataToSign;

    // BIM data
    private String birthDate;
    private String phoneNo;
    /**
     * For sending SMS or for getting transactionInfo for BIM action.
     */
    private String transactionReference;

    // Used for webclient in case redirect at end instead of JS callback
    private String webClientRedirAtEndUrls;
    // When webclient start it points to error url, at end to this url
    private String okUrl;

    /**
     * Defines number of concurrent login,authentication or signing BankID sequence data the BankID APP to hold.
     */
    final static int MAX_SESSIONS = 25;

    final static SizeRestrictedMap<String, SessionSequenceInfo> sidToBankIDSession = new SizeRestrictedMap<>(MAX_SESSIONS);

    /**
     * For getting transactionData at end for WebClient
     */
    private String traceId;

    private String bankIDHelperInitParams;
    private String keyStrokedata;
//    private CertificateInfo certificateInfo;
    private String merchantReference;
    private List<byte[]> completePDFs;
    private boolean noGraphicFlag;
    private boolean userPressedCancel;

    public String getSid() {
        return sid;
    }

    public SessionSequenceInfo setDataToSign(final List<ParallelOrSerialDocumentToSign> dataToSign) {
        this.dataToSign = dataToSign;
        return this;
    }

    public List<ParallelOrSerialDocumentToSign> getDataToSign() {
        return dataToSign;
    }

    public SessionSequenceInfo setTraceId(final String traceId) {
        this.traceId = traceId;
        return this;
    }

    public String getTraceId() {
        return traceId;
    }

    public SessionSequenceInfo setBankIDHelperInitParams(String bankIDHelperInitParams) {
        this.bankIDHelperInitParams = bankIDHelperInitParams;
        return this;
    }

    public String getBankIDHelperInitParams() {
        return bankIDHelperInitParams;
    }

    public SessionSequenceInfo setKeyStrokedata(String keyStrokedata) {
        this.keyStrokedata = keyStrokedata;
        return this;
    }

    public Optional<String> getKeyStrokedata() {
        return emptyIfBlank(keyStrokedata);
    }

    public SessionSequenceInfo setOkUrl(String okUrl) {
        this.okUrl = okUrl;
        return this;
    }

    public String getOkUrl() {
        return okUrl;
    }

    public int getPadessLength() {
        return completePDFs == null ? 0 : completePDFs.size();
    }

    public String getMerchantReference() {
        return merchantReference;
    }

    public SessionSequenceInfo setMerchantReference(String merchantReference) {
        this.merchantReference = merchantReference;
        return this;
    }

    public SessionSequenceInfo setCompletePDFs(List<byte[]> completePDFs) {
        this.completePDFs = completePDFs;
        return this;
    }

    public List<byte[]> getCompletePDFs() {
        return completePDFs;
    }

    public boolean isSelfAssemblingValidation() {
        return selfAssemblerValidation;
    }

    public SessionSequenceInfo setNoGraphicFlag(boolean noGraphicFlag) {
        this.noGraphicFlag = noGraphicFlag;
        return this;
    }

    public boolean noGraphicFlagIsSet() {
        return noGraphicFlag;
    }

    public void setUserPressedCancel(boolean userPressedCancel) {
        this.userPressedCancel = userPressedCancel;
    }

    public boolean getUserPressedCancel() {
        return userPressedCancel;
    }

    /**
     * The various state clients implementing login, authentication or signing
     * using BankID WebClient or BankID mobile may have.
     */
    public enum NextBankIDStep {
        WAITING_TO_BE_STARTED,
        WAITING_INIT,
        WAITING_VERIFY,
        FINISHED
    }

    public SessionSequenceInfo(NextBankIDStep nextBankIDStep) {
        this.nextStep = nextBankIDStep;
        this.sid = Utils.nextSid();
        sidToBankIDSession.put(sid, this);
    }

    /**
     * @param sid the value of the "sid" parameter key received from BankID callback
     */
    public static SessionSequenceInfo getSessionData(String sid) {
        return sidToBankIDSession.get(sid);
    }


    public SessionSequenceInfo setNextBankIDStep(NextBankIDStep nextStep) {
        this.nextStep = nextStep;
        return this;
    }

    public NextBankIDStep getNextStep() {
        return nextStep;
    }

    public String getErrorCode() {
        return errorCode;
    }

    public SessionSequenceInfo setErrorCode(String errorCode) {
        this.errorCode = errorCode;
        return this;
    }

    public String getErrorMsg() {
        return errorMsg;
    }

    public SessionSequenceInfo setErrorMsg(String errorMsg) {
        this.errorMsg = errorMsg;
        return this;
    }

    /**
     * Getter for webClientRedirAtEndUrls.
     *
     * @return the held webClientRedirAtEndUrls.
     */
    public String getWebClientRedirAtEndUrls() {
        return this.webClientRedirAtEndUrls;
    }

    /**
     * Setter for webClientRedirAtEndUrls.
     *
     * @param webClientRedirAtEndUrls what to set.
     * @return this, for chaining setters.
     */
    public SessionSequenceInfo setWebClientRedirAtEndUrls(String webClientRedirAtEndUrls) {
        this.webClientRedirAtEndUrls = webClientRedirAtEndUrls;
        return this;
    }
    public SessionSequenceInfo setSsn(String ssn) {
        this.ssn = ssn;
        return this;
    }

    public SessionSequenceInfo setBirthDate(String birthDate) {
        this.birthDate = birthDate;
        return this;
    }

    public SessionSequenceInfo setPhoneNo(String phoneNo) {
        this.phoneNo = phoneNo;
        return this;
    }

    public Optional<String> getSsn() { return emptyIfBlank(ssn); }

    public String getBirthDate() {
        return birthDate;
    }

    public String getPhoneNo() {
        return phoneNo;
    }
    public String getTransactionReference() {
        return transactionReference;
    }

    /**
     * Needed for BankID mobil sending sms
     *
     * @param transactionReference the transactionref received from step 2
     * @return this
     */
    public SessionSequenceInfo setTransactionReference(String transactionReference) {
        this.transactionReference = transactionReference;
        return this;
    }

    public boolean isSerialSigning() {
        return dataToSign.stream().anyMatch(documentToSign -> documentToSign.kind == ParallelOrSerialDocumentToSign.Kind.BSERIAL);
    }
    /**
     * Necessary for serial signing during initTransaction
     */
    private int signingKind;
    private boolean selfAssemblerSeals;
    private boolean selfAssemblerValidation;
    private boolean useNewCMSAndOCSPFormats;
    private byte[] visualMerchantSeal;
    private byte[] visualEndUserSeal;

    public InputStream getVisualMerchantSeal() {
        return new ByteArrayInputStream(visualMerchantSeal);
    }

    public SessionSequenceInfo setVisualMerchantSeal(byte[] visualMerchantSeal) {
        this.visualMerchantSeal = visualMerchantSeal;
        return this;
    }

    public InputStream getVisualEndUserSeal() {
        return new ByteArrayInputStream(visualEndUserSeal);
    }

    public SessionSequenceInfo setVisualEndUserSeal(byte[] visualEndUserSeal) {
        this.visualEndUserSeal = visualEndUserSeal;
        return this;
    }

    public Boolean getUseNewCMSAndOCSPFormats() {
        return useNewCMSAndOCSPFormats;
    }

    public SessionSequenceInfo setUseNewCMSAndOCSPFormats(boolean useNewCMSAndOCSPFormats) {
        this.useNewCMSAndOCSPFormats = useNewCMSAndOCSPFormats;
        return this;
    }

    public Boolean isSelfAssemblerSeals() {
        return selfAssemblerSeals;
    }

    public SessionSequenceInfo setSelfAssemblerSeals(boolean selfAssemblerSeals) {
        this.selfAssemblerSeals = selfAssemblerSeals;
        return this;
    }

    public SessionSequenceInfo setSelfAssemblerValidation(boolean selfAssemblerValidation) {
        this.selfAssemblerValidation = selfAssemblerValidation;
        return this;
    }

    public SessionSequenceInfo setSigningKind(int signingKind) {
        this.signingKind = signingKind;
        return this;
    }

    public int getSigningKind() {
        return signingKind;
    }



    static Optional<String> emptyIfBlank(String x) {
        return StringUtil.isBlank(x) ? Optional.empty() : Optional.of(x);
    }
}
