package bid.ex.webclient;

import org.apache.log4j.Logger;
import bid.ex.BankIDConfig;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Objects;

import static java.lang.String.format;

public class BankIDWebClientCallbackFilter implements Filter {
    private static final Logger log = Logger.getLogger(BankIDWebClientCallbackFilter.class);

    @Override
    public void init(final FilterConfig filterConfig) throws ServletException {
    }

    @Override
    public void doFilter(final ServletRequest request, final ServletResponse servletResponse, final FilterChain chain) throws IOException, ServletException {
        // See:  https://developer.mozilla.org/en/docs/HTTP/Access_control_CORS

       log.debug("WebClient: Filters request");
        if (servletResponse instanceof HttpServletResponse) {
            final HttpServletResponse httpServletResponse = (HttpServletResponse) servletResponse;
            setStandardNoCacheHeaders(httpServletResponse);
            setContentSecurityPolicy(httpServletResponse);

            if (request instanceof HttpServletRequest) {
                String origin = ((HttpServletRequest) request).getHeader("Origin");
                log.debug("WebClient: " + origin);

                if (Objects.equals(origin, BankIDConfig.getBankIDCSAndOriginatorUrl())) {
                    log.debug("WebClient: Request has correct origin, allows it");
                    httpServletResponse.setHeader("Access-Control-Allow-Origin", BankIDConfig.getBankIDCSAndOriginatorUrl());

                    // If a common httpSession is needed between the bankID clients iFrame and the parent window you need to have
                    // Access-Control-Allow-Credentials:true" and a corresponding BankIDWebClientInfo.setWithCredentials(true) in step1
                    // If not, the session cookie will not be shared.

                    httpServletResponse.setHeader("Access-Control-Allow-Credentials", "true");
                }
            }
        }
        chain.doFilter(request, servletResponse);
    }

    private void setStandardNoCacheHeaders(final HttpServletResponse response) {
        response.setHeader("X-UA-Compatible", "IE=edge");
        response.setHeader("Expires", "0");
        response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
        response.setHeader("Pragma", "no-cache");
    }

    @Override
    public void destroy() {

    }

    /**
     * Sets the content security policy header to allow the bankid helper and bankid client to load from bankid site.
     * <p> The HTTP header is 'X-Content-Security-Policy' for Firefox until version 23 and IE10&11,
     * and 'X-Webkit-CSP' for Chrome until version 25 and Safari until version 7.
     * </p><p>
     * 'Content-Security-Policy' is the official W3C defined header, used by Chrome version 25 and later,
     * Firefox version 23 and later, and Safari 7 and later.</p>
     */
    private void setContentSecurityPolicy(final HttpServletResponse response) {
        String CoSPValue = format("default-src 'self'; frame-src 'self' %1$s;script-src 'self' %1$s;style-src 'self' %1$s;", BankIDConfig.getBankIDCSAndOriginatorUrl());
        response.setHeader("X-Webkit-CSP", CoSPValue);
        response.setHeader("X-Content-Security-Policy", CoSPValue);
        response.setHeader("Content-Security-Policy", CoSPValue);
    }
}
